@@ -19,7 +19,7 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: server.h,v 1.3 2006/07/11 16:12:27 hanke Exp $
+ * $Id: server.h,v 1.3 2006-07-11 16:12:27 hanke Exp $
  */
 
 #ifndef SERVER_H
@@ -29,13 +29,11 @@
 int serve(int fd);
 
 /* Switches `receiving data' mode on and off for specified client */
-int server_data_on(int fd);
+void server_data_on(int fd);
 void server_data_off(int fd);
 
 /* Put a message into Dispatcher's queue */
-int queue_message(TSpeechDMessage *new, int fd, int history_flag,
-		  EMessageType type, int reparted);
-
+int queue_message(TSpeechDMessage * new, int fd, int history_flag,
+		  SPDMessageType type, int reparted);
 
 #endif
-
