@@ -1,6 +1,6 @@
 /*
  * speechd.c - Speech Dispatcher server program
- *  
+ *
  * Copyright (C) 2001, 2002, 2003, 2006, 2007 Brailcom, o.p.s.
  *
  * This is free software; you can redistribute it and/or modify it
@@ -58,7 +58,25 @@ void destroy_pid_file();
 /* Server socket file descriptor */
 int server_socket;
 
-void speechd_load_configuration(int sig);
+GMainLoop *main_loop = NULL;
+gint server_timeout_source = 0;
+
+int client_count = 0;
+
+static gboolean speechd_client_terminate(gpointer key, gpointer value, gpointer user);
+static gboolean speechd_reload_dead_modules(gpointer user_data);
+static gboolean speechd_load_configuration(gpointer user_data);
+static gboolean speechd_quit(gpointer user_data);
+
+static gboolean server_process_incoming (gint          fd,
+				  GIOCondition  condition,
+				  gpointer      data);
+
+static gboolean client_process_incoming (gint          fd,
+				  GIOCondition  condition,
+				  gpointer      data);
+
+void check_client_count(void);
 
 #ifdef __SUNPRO_C
 /* Added by Willie Walker - daemon is a gcc-ism
@@ -124,7 +142,7 @@ void fatal_error(void)
 	i++;
 }
 
-/* Logging messages, level of verbosity is defined between 1 and 5, 
+/* Logging messages, level of verbosity is defined between 1 and 5,
  * see documentation */
 void MSG2(int level, char *kind, char *format, ...)
 {
@@ -156,7 +174,7 @@ void MSG2(int level, char *kind, char *format, ...)
 					fprintf(logfile, "[%s : %d] speechd: ",
 						tstr, (int)tv.tv_usec);
 					//            fprintf(logfile, "[test : %d] speechd: ",
-					//                     (int) tv.tv_usec); 
+					//                     (int) tv.tv_usec);
 				}
 				if (custom_log) {
 					fprintf(custom_logfile,
@@ -349,7 +367,6 @@ int speechd_connection_new(int server_socket)
 	}
 
 	/* We add the associated client_socket to the descriptor set. */
-	FD_SET(client_socket, &readfds);
 	if (client_socket > SpeechdStatus.max_fd)
 		SpeechdStatus.max_fd = client_socket;
 	MSG(4, "Adding client on fd %d", client_socket);
@@ -363,7 +380,6 @@ int speechd_connection_new(int server_socket)
 		    "Error: Failed to create a record in fd_settings for the new client");
 		if (SpeechdStatus.max_fd == client_socket)
 			SpeechdStatus.max_fd--;
-		FD_CLR(client_socket, &readfds);
 		return -1;
 	}
 	new_fd_set->fd = client_socket;
@@ -378,8 +394,15 @@ int speechd_connection_new(int server_socket)
 	g_hash_table_insert(fd_settings, p_client_uid, new_fd_set);
 	g_hash_table_insert(fd_uid, p_client_socket, p_client_uid2);
 
+	new_fd_set->fd_source = g_unix_fd_add(client_socket, G_IO_IN, client_process_incoming, NULL);
+
 	MSG(4, "Data structures for client on fd %d created", client_socket);
+
+	client_count++;
+	check_client_count();
+
 	return 0;
+
 }
 
 int speechd_connection_destroy(int fd)
@@ -394,6 +417,7 @@ int speechd_connection_destroy(int fd)
 	if (fdset_element != NULL) {
 		fdset_element->fd = -1;
 		fdset_element->active = 0;
+		g_source_remove(fdset_element->fd_source);
 		/* The fdset_element will be freed and removed from the
 		   hash table as soon as the client no longer has any
 		   message in the queues, check out the speak() function */
@@ -413,16 +437,18 @@ int speechd_connection_destroy(int fd)
 		if (SPEECHD_DEBUG)
 			DIE("Can't close file descriptor associated to this client");
 
-	FD_CLR(fd, &readfds);
 	if (fd == SpeechdStatus.max_fd)
 		SpeechdStatus.max_fd--;
 
 	MSG(4, "Connection closed");
 
+	client_count--;
+	check_client_count();
+
 	return 0;
 }
 
-gboolean speechd_client_terminate(gpointer key, gpointer value, gpointer user)
+static gboolean speechd_client_terminate(gpointer key, gpointer value, gpointer user)
 {
 	TFDSetElement *set;
 
@@ -504,13 +530,14 @@ void speechd_module_nodebug(gpointer data, gpointer user_data)
 	return;
 }
 
-void speechd_reload_dead_modules(int sig)
+static gboolean speechd_reload_dead_modules(gpointer user_data)
 {
 	/* Reload dead modules */
 	g_list_foreach(output_modules, speechd_modules_reload, NULL);
 
 	/* Make sure there aren't any more child processes left */
 	while (waitpid(-1, NULL, WNOHANG) > 0) ;
+	return TRUE;
 }
 
 void speechd_modules_debug(void)
@@ -538,6 +565,7 @@ void speechd_options_init(void)
 	SpeechdOptions.localhost_access_only_set = 0;
 	SpeechdOptions.pid_file = NULL;
 	SpeechdOptions.conf_file = NULL;
+	SpeechdOptions.module_dir = MODULEBINDIR;
 	SpeechdOptions.runtime_speechd_dir = NULL;
 	SpeechdOptions.log_dir = NULL;
 	SpeechdOptions.log_dir_set = 0;
@@ -614,7 +642,8 @@ void speechd_init()
 
 	if (SpeechdOptions.log_dir == NULL) {
 		SpeechdOptions.log_dir =
-		    g_strdup_printf("%s/log/", SpeechdOptions.runtime_speechd_dir);
+		    g_strdup_printf("%s/log/",
+				    SpeechdOptions.runtime_speechd_dir);
 		mkdir(SpeechdOptions.log_dir, S_IRWXU);
 		if (!SpeechdOptions.debug_destination) {
 			SpeechdOptions.debug_destination =
@@ -627,7 +656,7 @@ void speechd_init()
 	/* Load configuration from the config file */
 	MSG(4, "Reading Speech Dispatcher configuration from %s",
 	    SpeechdOptions.conf_file);
-	speechd_load_configuration(0);
+	speechd_load_configuration(NULL);
 
 	logging_init();
 
@@ -643,7 +672,7 @@ void speechd_init()
 	last_p5_block = NULL;
 }
 
-void speechd_load_configuration(int sig)
+static gboolean speechd_load_configuration(gpointer user_data)
 {
 	configfile_t *configfile = NULL;
 	GList *detected_modules = NULL;
@@ -684,16 +713,18 @@ void speechd_load_configuration(int sig)
 		/* We need to load modules here, since this is called both by speechd_init
 		 * and to handle SIGHUP. */
 		if (module_number_of_requested_modules() < 2) {
-			detected_modules = detect_output_modules(MODULEBINDIR);
+			detected_modules = detect_output_modules(SpeechdOptions.module_dir);
 			while (detected_modules != NULL) {
 				char **parameters = detected_modules->data;
-				module_add_load_request(parameters[0], parameters[1],
-							parameters[2], parameters[3]);
+				module_add_load_request(parameters[0],
+							parameters[1],
+							parameters[2],
+							parameters[3]);
 				g_free(detected_modules->data);
 				detected_modules->data = NULL;
 				detected_modules =
-					g_list_delete_link(detected_modules,
-							   detected_modules);
+				    g_list_delete_link(detected_modules,
+						       detected_modules);
 			}
 		}
 
@@ -703,47 +734,14 @@ void speechd_load_configuration(int sig)
 	}
 
 	free_config_options(spd_options, &spd_num_options);
+
+	return TRUE;
 }
 
-void speechd_quit(int sig)
+static gboolean speechd_quit(gpointer user_data)
 {
-	int ret;
-
-	MSG(1, "Terminating...");
-
-	MSG(2, "Closing open connections...");
-	/* We will browse through all the connections and close them. */
-	g_hash_table_foreach_remove(fd_settings, speechd_client_terminate,
-				    NULL);
-	g_hash_table_destroy(fd_settings);
-
-	MSG(4, "Closing speak() thread...");
-	ret = pthread_cancel(speak_thread);
-	if (ret != 0)
-		FATAL("Speak thread failed to cancel!\n");
-
-	ret = pthread_join(speak_thread, NULL);
-	if (ret != 0)
-		FATAL("Speak thread failed to join!\n");
-
-	MSG(2, "Closing open output modules...");
-	/*  Call the close() function of each registered output module. */
-	g_list_foreach(output_modules, speechd_modules_terminate, NULL);
-	g_list_free(output_modules);
-
-	MSG(2, "Closing server connection...");
-	if (close(server_socket) == -1)
-		MSG(2, "close() failed: %s", strerror(errno));
-	FD_CLR(server_socket, &readfds);
-
-	MSG(4, "Removing pid file");
-	destroy_pid_file();
-
-	fflush(NULL);
-
-	MSG(2, "Speech Dispatcher terminated correctly");
-
-	exit(0);
+	g_main_loop_quit(main_loop);
+	return FALSE;
 }
 
 /* --- PID FILES --- */
@@ -917,20 +915,80 @@ int make_inet_socket(const int port)
 	return server_socket;
 }
 
+gboolean server_process_incoming (gint          fd,
+				  GIOCondition  condition,
+				  gpointer      data)
+{
+	int ret;
+
+	ret = speechd_connection_new(fd);
+	if (ret != 0) {
+		MSG(2, "Error: Failed to add new client!");
+		if (SPEECHD_DEBUG) {
+			FATAL("Failed to add new client");
+		}
+	}
+
+	return TRUE;
+}
+
+gboolean client_process_incoming (gint          fd,
+				  GIOCondition  condition,
+				  gpointer      data)
+{
+	int ret;
+	int nread;
+
+	ioctl(fd, FIONREAD, &nread);
+
+	if (nread == 0) {
+		/* client has gone */
+		ret = speechd_connection_destroy(fd);
+		if (ret != 0) {
+			MSG(2, "Error: Failed to close the client!");
+		}
+		return FALSE;
+	}
+
+	/* client sends some commands or data */
+	if (serve(fd) == -1) {
+		MSG(2, "Error: Failed to serve client on fd %d!", fd);
+	}
+
+	return TRUE;
+}
+
+void check_client_count(void)
+{
+	if (client_count <= 0
+	    && SpeechdOptions.server_timeout > 0) {
+		MSG(4, "Currently no clients connected, enabling shutdown timer.");
+		server_timeout_source = 
+		                        g_timeout_add_seconds(
+		                        SpeechdOptions.server_timeout,
+		                        speechd_quit, NULL);
+	} else {
+	MSG(4, "Clients connected, disabling shutdown timer.");
+		g_source_remove(server_timeout_source);
+	}
+}
+
 /* --- MAIN --- */
 
 int main(int argc, char *argv[])
 {
-	fd_set testfds;
-	int fd;
 	int ret;
 	/* Autospawn helper variables */
 	char *spawn_communication_method = NULL;
 	int spawn_port = 0;
 	char *spawn_socket_path = NULL;
 
-	/* Initialize threading and thread safety in Glib */
+	/* Initialize threading and thread safety in Glib
+	 * Deprecated in 2.32
+	 */
+#ifndef GLIB_VERSION_2_32
 	g_thread_init(NULL);
+#endif
 
 	/* Strip all permisions for 'others' from the files created */
 	umask(007);
@@ -973,38 +1031,52 @@ int main(int argc, char *argv[])
 	MSG(1, "Speech Dispatcher " VERSION " starting");
 
 	/* By default, search for configuration options in $XDG_CONFIG_HOME/speech-dispatcher
-           and sockets and pid files in $XDG_RUNTIME_DIR/speech-dispatcher */
+	   and sockets and pid files in $XDG_RUNTIME_DIR/speech-dispatcher */
 	{
 		const char *user_runtime_dir;
 		const char *user_config_dir;
+		char *test_speechd_conf_file = NULL;
 
-		user_runtime_dir = g_get_user_runtime_dir ();
-		user_config_dir = g_get_user_config_dir ();
+		user_runtime_dir = g_get_user_runtime_dir();
+		user_config_dir = g_get_user_config_dir();
 
 		/* Setup a speechd-dispatcher directory or create a new one */
-		SpeechdOptions.runtime_speechd_dir = g_strdup_printf("%s/speech-dispatcher",
-								     user_runtime_dir);
+		SpeechdOptions.runtime_speechd_dir =
+		    g_strdup_printf("%s/speech-dispatcher", user_runtime_dir);
 		MSG(4, "Trying to find %s", SpeechdOptions.runtime_speechd_dir);
-		g_mkdir_with_parents(SpeechdOptions.runtime_speechd_dir, S_IRWXU);
-		MSG(4, "Using directory: %s for pidfile and logging", SpeechdOptions.runtime_speechd_dir);
+		g_mkdir_with_parents(SpeechdOptions.runtime_speechd_dir,
+				     S_IRWXU);
+		MSG(4, "Using directory: %s for pidfile and logging",
+		    SpeechdOptions.runtime_speechd_dir);
 		/* Pidfile */
 		if (SpeechdOptions.pid_file == NULL) {
 			/* If no pidfile path specified on command line, use default local dir */
-			SpeechdOptions.pid_file = g_strdup_printf ("%s/pid/speech-dispatcher.pid",
-								   SpeechdOptions.runtime_speechd_dir);
-			g_mkdir(g_path_get_dirname (SpeechdOptions.pid_file), S_IRWXU);
+			SpeechdOptions.pid_file =
+			    g_strdup_printf("%s/pid/speech-dispatcher.pid",
+					    SpeechdOptions.runtime_speechd_dir);
+			g_mkdir(g_path_get_dirname(SpeechdOptions.pid_file),
+				S_IRWXU);
 		}
 		/* Config file */
 		if (SpeechdOptions.conf_dir == NULL) {
 			/* If no conf_dir was specified on command line, try default local config dir */
-			SpeechdOptions.conf_dir = g_build_filename(user_config_dir, "speech-dispatcher", NULL);
-			if (!g_file_test (SpeechdOptions.conf_dir, G_FILE_TEST_IS_DIR)) {
-				/* If the local confiration dir doesn't exist, read the global configuration */
+			SpeechdOptions.conf_dir =
+			    g_build_filename(user_config_dir,
+					     "speech-dispatcher", NULL);
+			test_speechd_conf_file =
+			    g_build_filename(SpeechdOptions.conf_dir,
+					     "speechd.conf", NULL);
+			if (!g_file_test
+			    (test_speechd_conf_file, G_FILE_TEST_IS_REGULAR)) {
+				/* If the local configuration file doesn't exist, read the global configuration */
 				if (strcmp(SYS_CONF, ""))
-					SpeechdOptions.conf_dir = g_strdup(SYS_CONF);
+					SpeechdOptions.conf_dir =
+					    g_strdup(SYS_CONF);
 				else
-					SpeechdOptions.conf_dir = g_strdup ("/etc/speech-dispatcher/");
+					SpeechdOptions.conf_dir =
+					    g_strdup("/etc/speech-dispatcher/");
 			}
+			g_free(test_speechd_conf_file);
 		}
 		SpeechdOptions.conf_file =
 		    g_strdup_printf("%s/speechd.conf", SpeechdOptions.conf_dir);
@@ -1052,12 +1124,6 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
-	/* Register signals */
-	(void)signal(SIGINT, speechd_quit);
-	(void)signal(SIGHUP, speechd_load_configuration);
-	(void)signal(SIGPIPE, SIG_IGN);
-	(void)signal(SIGUSR1, speechd_reload_dead_modules);
-
 	speechd_init();
 
 	/* Handle socket_path 'default' */
@@ -1113,24 +1179,22 @@ int main(int argc, char *argv[])
 							    spawn_port);
 							exit(1);
 						}
-				} else
-				    if (!strcmp
-					(SpeechdOptions.communication_method,
-					 "unix_socket")) {
+				} else if (!strcmp
+					   (SpeechdOptions.communication_method,
+					    "unix_socket")) {
 					/* Check socket name */
 					if (spawn_socket_path)
 						if (strcmp
 						    (spawn_socket_path,
-						     SpeechdOptions.
-						     socket_path)) {
+						     SpeechdOptions.socket_path))
+						{
 							MSG(-1,
 							    "Autospawn failed: Mismatch in socket names. The server "
 							    "is configured to provide a socket interface in %s, but the "
 							    "client requests a different path: %s. This is most probably "
 							    "due to the client application configuration or the value of "
 							    "the SPEECHD_ADDRESS environment variable.",
-							    SpeechdOptions.
-							    socket_path,
+							    SpeechdOptions.socket_path,
 							    spawn_socket_path);
 							exit(1);
 						}
@@ -1173,70 +1237,68 @@ int main(int argc, char *argv[])
 			return -1;
 	}
 
+	/* Set up the main loop and register signals */
+        main_loop = g_main_loop_new(g_main_context_default(), FALSE);
+	g_unix_signal_add(SIGINT, speechd_quit, NULL);
+	g_unix_signal_add(SIGTERM, speechd_quit, NULL);
+	g_unix_signal_add(SIGHUP, speechd_load_configuration, NULL);
+	g_unix_signal_add(SIGUSR1, speechd_reload_dead_modules, NULL);
+	(void)signal(SIGPIPE, SIG_IGN);
+
 	MSG(4, "Creating new thread for speak()");
 	ret = pthread_create(&speak_thread, NULL, speak, NULL);
 	if (ret != 0)
 		FATAL("Speak thread failed!\n");
 
-	FD_ZERO(&readfds);
-	FD_SET(server_socket, &readfds);
 	SpeechdStatus.max_fd = server_socket;
 
+	g_unix_fd_add(server_socket, G_IO_IN,
+		      server_process_incoming, NULL);
+
 	/* Now wait for clients and requests. */
 	MSG(1, "Speech Dispatcher started and waiting for clients ...");
-	while (1) {
-		testfds = readfds;
 
-		if (select
-		    (FD_SETSIZE, &testfds, (fd_set *) 0, (fd_set *) 0,
-		     NULL) >= 1) {
-			/* Once we know we've got activity,
-			 * we find which descriptor it's on by checking each in turn using FD_ISSET.
-			 TODO: This is a hack. We should not be browsing all possible socket numbers
-			 but only those which are connected.
-			 */
-			for (fd = 0;
-			     fd <= SpeechdStatus.max_fd && fd < FD_SETSIZE;
-			     fd++) {
-				if (FD_ISSET(fd, &testfds)) {
-					MSG(4, "Activity on fd %d ...", fd);
+	check_client_count();
 
-					if (fd == server_socket) {
-						/* server activity (new client) */
-						ret =
-						    speechd_connection_new
-						    (server_socket);
-						if (ret != 0) {
-							MSG(2,
-							    "Error: Failed to add new client!");
-							if (SPEECHD_DEBUG)
-								FATAL
-								    ("Failed to add new client");
-						}
-					} else {
-						/* client activity */
-						int nread;
-						ioctl(fd, FIONREAD, &nread);
+	g_main_loop_run(main_loop);
 
-						if (nread == 0) {
-							/* client has gone */
-							speechd_connection_destroy
-							    (fd);
-							if (ret != 0)
-								MSG(2,
-								    "Error: Failed to close the client!");
-						} else {
-							/* client sends some commands or data */
-							if (serve(fd) == -1)
-								MSG(2,
-								    "Error: Failed to serve client on fd %d!",
-								    fd);
-						}
-					}
-				}
-			}
-		}
-	}
+	MSG(1, "Terminating...");
+
+	MSG(2, "Closing open connections...");
+	/* We will browse through all the connections and close them. */
+	g_hash_table_foreach_remove(fd_settings, speechd_client_terminate,
+				    NULL);
+	g_hash_table_destroy(fd_settings);
+
+	MSG(4, "Closing speak() thread...");
+	ret = pthread_cancel(speak_thread);
+	if (ret != 0)
+		FATAL("Speak thread failed to cancel!\n");
+
+	ret = pthread_join(speak_thread, NULL);
+	if (ret != 0)
+		FATAL("Speak thread failed to join!\n");
+
+	MSG(2, "Closing open output modules...");
+	/*  Call the close() function of each registered output module. */
+	g_list_foreach(output_modules, speechd_modules_terminate, NULL);
+	g_list_free(output_modules);
+
+	MSG(2, "Closing server connection...");
+	if (close(server_socket) == -1)
+		MSG(2, "close() failed: %s", strerror(errno));
+
+	MSG(4, "Removing pid file");
+	destroy_pid_file();
+
+	fflush(NULL);
+
+	g_main_loop_unref(main_loop);
+	main_loop = NULL;
+
+	MSG(2, "Speech Dispatcher terminated correctly");
+
+	exit(0);
 }
 
 void check_locked(pthread_mutex_t * lock)
